#############################################################################
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!#
#!!!!!!!!!!!!!!!!!!!!!!!!!FUNCTIONS!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!#
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!#
#############################################################################

#############################################################################
# biovarsClimateData                                                        #
# time_sclice: possible time slices = 'GIS11', 'H4', 'H5'
#############################################################################

biovarsClimateData <- function(time_slice){
  pr <-  stack(list.files(path=paste(cdf, '/', time_slice, '/pr', sep=''), pattern = 'tif', full.names = TRUE))
  tn <-  stack(list.files(path=paste(cdf, '/', time_slice, '/tn', sep=''), pattern = 'tif', full.names = TRUE))
  tx <-  stack(list.files(path=paste(cdf, '/', time_slice, '/tx', sep=''), pattern = 'tif', full.names = TRUE))
  
  objects<-biovars(prec=pr,tmin=tn,tmax=tx)
  
  for (i in 1:19) {
    writeRaster(objects[[i]],paste(cdf, '/', time_slice, '/bio/bio',i,".tif",sep=""), format='GTiff', overwrite=T)
  }
  return(TRUE)
}

#############################################################################
# mosaicList                                                                #
# Call the function mosaic with a list of raster files
#############################################################################
mosaicList <- function(rasList){
  
  #Internal function to make a list of raster objects from list of files.
  ListRasters <- function(list_names) {
    raster_list <- list() # initialise the list of rasters
    for (i in 1:(length(list_names))){ 
      grd_name <- list_names[i] # list_names contains all the names of the images in .grd format
      raster_file <- raster::raster(grd_name)
    }
    raster_list <- append(raster_list, raster_file) # update raster_list at each iteration
  }
  
  #convert every raster path to a raster object and create list of the results
  raster.list <-sapply(rasList, FUN = ListRasters)
  
  # edit settings of the raster list for use in do.call and mosaic
  names(raster.list) <- NULL
  #####This function deals with overlapping areas
  raster.list$fun <- sum
  
  #run do call to implement mosaic over the list of raster objects.
  mos <- do.call(raster::mosaic, raster.list)
  
  #set crs of output
  crs(mos) <- crs(x = raster(rasList[1]))
  return(mos)
}